#include <WiFi.h>
#include <HTTPClient.h>
#include "camera.h"
#include "ESP_I2S.h"

// WiFi配置
const char* ssid = "**************";
const char* password = "**************";
const char* serverURL = "你的服务器IP:5000/upload";
const int txPin = 44;
I2SClass i2s1;
const int WAV_BUFFER_SIZE = 1024 * 500; // 增大缓冲区至30KB
uint8_t *wav_buffer = nullptr;
uint8_t bs=0;
uint8_t bs2=0;
uint8_t bs3=0;
camera_fb_t *fb;
HTTPClient http;
int httpCode=0;
const int debounceTime = 5; // 消抖时间，单位为毫秒
void setup() {
  //Serial.begin(115200);
  pinMode(txPin, INPUT);
  pinMode(3, OUTPUT);
  digitalWrite(3,LOW);
  // 初始化摄像头
  if (!initCamera()) {
    //Serial.println("Camera init failed!");
    while(1);
  }

  // 连接WiFi
  WiFi.begin(ssid, password);
  while (WiFi.status() != WL_CONNECTED) {
    delay(10);
    //Serial.print(".");
  }
  //Serial.println("\nWiFi connected");

  // 初始化音频
  i2s1.setPins(45, 46, 42); // BCLK, LRC, DIN
  if (!i2s1.begin(I2S_MODE_STD, 16000, I2S_DATA_BIT_WIDTH_16BIT, I2S_SLOT_MODE_MONO)) {
    //Serial.println("I2S初始化失败!");
  }

  // 预分配音频缓冲区
  wav_buffer = (uint8_t*)malloc(WAV_BUFFER_SIZE);
  if (!wav_buffer) {
    //Serial.println("内存分配失败!");
  }
}
void printData(uint8_t* data, int length) {
  for (int i = 0; i < length; i++) {
    Serial.print(data[i], HEX); // 以十六进制形式打印每个字节
    Serial.print(" ");
  }
  Serial.println();
}
void parseWavHeader(uint8_t* data) {
  //printData(data,WAV_BUFFER_SIZE);
  // 简化的WAV头解析（偏移44字节）
  if (memcmp(data, "RIFF", 4) == 0 && memcmp(&data[8], "WAVE", 4) == 0) {
    //Serial.println("检测到标准WAV头，跳过44字节");
    memmove(data, data + 44, WAV_BUFFER_SIZE - 44);
  }
}
void playwav(int audioSize,uint8_t* Tbs,uint8_t Lbs){
  
  if (audioSize > 0 && audioSize <= WAV_BUFFER_SIZE) {
      *Tbs=Lbs;
      int bytesRead = http.getStream().readBytes(wav_buffer, audioSize);
      if (bytesRead != audioSize) {
        //Serial.printf("读取不完整: %d/%d\n", bytesRead, audioSize);
      } else {
        parseWavHeader(wav_buffer); // 处理WAV头
        audioSize -= 44; // 调整有效数据大小
        
        // 分块播放（使用标准write方法）
        const int chunkSize = 512;
        for (int i = 0; i < audioSize; i += chunkSize) {
          int playSize = min(chunkSize, audioSize - i);
          size_t bytesWritten = i2s1.write(wav_buffer + i, playSize);
          if (bytesWritten != playSize) {
            //Serial.printf("I2S写入失败: %d/%d\n", bytesWritten, playSize);
          }
        }
        //Serial.println("播放完成");
      }
 }
}
void loop() {
  if(bs3==0){
    bs3=1;
    delay(5000);
    digitalWrite(3,HIGH);
    delay(100);
    digitalWrite(3,LOW);
    delay(100);
  }
  int txLevel = digitalRead(txPin);
  delay(debounceTime); // 简单的消抖
  int txLevelAfterDebounce = digitalRead(txPin);
  if (txLevel == txLevelAfterDebounce) { // 确认电平稳定
    if (txLevel == HIGH) {
      bs2=1;
      digitalWrite(3,HIGH);
      delay(200);
      digitalWrite(3,LOW);
      http.setTimeout(30000);
      http.begin(serverURL);
      http.addHeader("Content-Type", "text/plain");
      httpCode = http.POST("photo");
      if (httpCode == HTTP_CODE_OK) {
      // 获取音频响应
      int audioSize = http.getSize();
      //Serial.printf("收到音频大小: %d字节\n", audioSize);
      playwav(audioSize,&bs,0);
      http.end();
    } 
  }
  }

  if(bs2==1){
   if(bs==0){
      camera_fb_t *fb = esp_camera_fb_get();
      if (!fb) {
         //Serial.println("摄像头捕获失败");
         return;
      }
       http.setTimeout(30000);
       http.begin(serverURL);
       http.addHeader("Content-Type", "image/jpeg");
  
       //Serial.printf("发送图像数据大小: %d字节\n", fb->len);
       httpCode = http.POST(fb->buf, fb->len);
        esp_camera_fb_return(fb);
      }
      else{
      http.setTimeout(30000);
      http.begin(serverURL);
      http.addHeader("Content-Type", "text/plain");
      httpCode = http.POST("send");
      }
   if (httpCode == HTTP_CODE_OK) {
    // 获取音频响应
    int audioSize = http.getSize();
    //Serial.printf("收到音频大小: %d字节\n", audioSize);
    playwav(audioSize,&bs,1);
    
    } else {
      bs=0;
      bs2=0;
      // 清理资源
      http.end();
      digitalWrite(3,LOW);
      //Serial.printf("无效音频大小: %d（最大支持: %d）\n", audioSize, WAV_BUFFER_SIZE);
    }
  } else {
    bs=0;
    bs2=0;
    // 清理资源
    http.end();
    digitalWrite(3,LOW);
    //Serial.printf("HTTP错误码: %d\n", httpCode);
  }
  
  if(bs==0)
  {
    //bs2=1;
    //delay(20000);
  }
  delay(10);
  
}

